%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%                                %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%      DECLIPPING MAIN FILE      %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%                                %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% using toolbox LTFAT

close all
clearvars -except data_rec


%% input file settings

audio_file = 'a08_violin';    %  'a08_violin'
                              %  'a16_clarinet'
                              %  'a18_bassoon'
                              %  'a25_harp'	
                              %  'a35_glockenspiel'
                              %  'a41_celesta'
                              %  'a42_accordion'
                              %  'a58_guitar_sarasate'
                              %  'a60_piano_schubert'
                              %  'a66_wind_ensemble_stravinsky'
                                
fprintf(['Loading audio ''', audio_file , '.wav''\n']);
[data, fs] = audioread(['Sounds/' audio_file '.wav']);

% normalization
fprintf('Normalizing audio')
maxAbsVal = max(abs(data));
data = data/maxAbsVal;
fprintf('(The original maximum was %f)\n', maxAbsVal)

% signal length
param.Ls = length(data);

%% settings
fprintf('Setting up parameters\n')

% clipping threshold
param.theta = 0.3;     % set (symetric) clipping threshold

% DGT parameters
param.wtype = 'hann';  % window type
param.w = 8192;        % window length 
param.a = param.w / 4; % window shift
param.M = 8192;        % number of frequency channels     


param.replaceReliable = 1;  % after finishing all iterations, replace reliable samples in declipped signal with original signal

% weighting
param.weighting = 1; % enables weighting of the coefficients during the restoration
param.weighting_type = 'mpeg'; % options: 'ath', 'mpeg', 'par'
param.weights_assignment = 1; % options: 1, 2, 3
param.psychoacoustic_model_source = 'clipped'; % options: 'clipped', 'reconstructed', 'original'


%% clipping
fprintf('Generating clipped signal\n')
[data_clipped, param.masks] = hard_clip(data, -param.theta, param.theta); % clipping of original signal


%% construction of frame
fprintf('Creating the frame\n')

param.F = frame('dgtreal', {param.wtype, param.w}, param.a, param.M); % creating requested frame

param.F = frametight(param.F); % creating Parseval tight frame
param.F = frameaccel(param.F, param.Ls);  % precomputation for a fixed signal length

% display the framebounds
[A, B] = framebounds(param.F, param.Ls);
fprintf('Framebounds are %g and %g\n',A,B)
if abs(A/B-1) >= 10e-6
    warning('Frame is not tight!')
end

% construction of the weights
if param.weighting
    fprintf('Constructing the weights\n')
    switch param.weighting_type
        case {'ATH', 'ath'}
            param.weights = weights_ath(0, fs/2, floor(param.M/2)+1, param.F, param.Ls, param.weights_assignment);
        case {'MPEG', 'mpeg'}
            switch param.psychoacoustic_model_source
                case {'clipped', 'clip'}
                    model_source = data_clipped;
                case {'original', 'orig'}
                    model_source = data;
                case {'reconstructed', 'rec'}
                    if exist('data_rec', 'var')
                        model_source = data_rec;
                    else
                        error('The recostructed signal is not available. Please run the algorithm without weighting first to obtain the reconstructed signal.')
                    end
                otherwise
                    error('Invalid source for the psychoacoustic model!')
            end
            param.weights = weights_mpeg(param.w, param.M, param.a, param.F, model_source, fs, param.weights_assignment);
        case {'PAR', 'par'}
            param.weights = weights_triag(0, fs/2, floor(param.M/2)+1, param.F, param.Ls, fs/2, 2);
    end
else
    param.weights = 1;
end


%% Proximal algorithm
fprintf('\nStarting the proximal algorithm\n')

tic;

[data_rec, dSDR_process, objective_process] = douglas_rachford(data_clipped, param, data);

time = toc;

% replace reliable samples with original from clipped signal and compute error on reliable samples
if param.replaceReliable
    reliableDifference = norm(data_clipped(param.masks.Mr) - data_rec(param.masks.Mr));
    data_rec(param.masks.Mr) = data_clipped(param.masks.Mr);
    fprintf('l2-norm of difference on reliable samples after proximal algorithm is %4.3f \n', reliableDifference);
end


%% Evaluation of the result
fprintf('Computing final SDR and plotting some nice figures\n')

% time
fprintf('Result obtained in %4.3f seconds.\n', time);

% SDR
sdr_clip = sdr(data, data_clipped);
sdr_rec = sdr(data, data_rec);
dsdr = sdr_rec - sdr_clip;
fprintf('SDR of the clipped signal is %4.3f dB.\n', sdr_clip);
fprintf('SDR of the reconstructed signal is %4.3f dB.\n', sdr_rec);
fprintf('SDR improvement is %4.3f dB.\n', dsdr);

% Plot of used weights (for n-th window)
if param.weighting
    n = 11;
    f = linspace(0, fs/2, floor(param.M/2)+1);
    w_native = framecoef2native(param.F,param.weights);
    figure
    plot(f, w_native(:,n));
    xlabel('frequency (Hz)')
    ylabel('normalized weights (-)')
    xlim([0 fs/2]);
    legend_string = ['normalized weights for ' num2str(n) '-th window'];
    legend(legend_string)
end

% Plot of declpped signal
t = linspace(0, length(data)/fs, length(data));
figure
plot(t, data);
hold on
plot(t, data_rec);
plot(t, data_clipped);
xlabel('time (s)')
legend({'original', 'reconstructed', 'clipped'})

% Plot of dSDR and objective function
figure
len = length(dSDR_process(~isnan(dSDR_process)));
t = linspace(0, time, len);
yyaxis left
    p1 = plot(t,dSDR_process);
    hold on
    px1 = plot(t(1:100:len), dSDR_process(1:100:len), 'x');
    ylabel('{\Delta}SDR (dB)');
      
yyaxis right
    p2 = plot(t, objective_process);
    px2 = plot(t(1:100:len), objective_process(1:100:len), 'x');
    xlabel('time (s)');
    ylabel('Objective function');
    grid on




